@@ -3,7 +3,7 @@ package Params::ValidationCompiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.22';
+our $VERSION = '0.25';
 
 use Params::ValidationCompiler::Compiler;
 
@@ -21,7 +21,7 @@ no warnings 'once';
 ## use critic
 
 sub source_for {
-    return Params::ValidationCompiler::Compiler->new(@_)->source_for;
+    return Params::ValidationCompiler::Compiler->new(@_)->source;
 }
 
 1;
@@ -40,7 +40,7 @@ Params::ValidationCompiler - Build an optimized subroutine parameter validator o
 
 =head1 VERSION
 
-version 0.22
+version 0.25
 
 =head1 SYNOPSIS
 
@@ -130,7 +130,7 @@ If you pass an arrayref and C<named_to_list> is false, the validator will
 expect positional params. Each element of the C<params> arrayref should be a
 parameter spec.
 
-If you pass an arrayref and C<named_to_list> is false, the validator will
+If you pass an arrayref and C<named_to_list> is true, the validator will
 expect named params, but will return a list of values. In this case the
 arrayref should contain a I<list> of key/value pairs, where parameter names
 are the keys and the specs are the values.
@@ -173,12 +173,12 @@ must be values of the specified type.
 =item * named_to_list
 
 If this is true, the generated subroutine will expect a list of key-value
-pairs or a hashref and it will return a list containing only the values.
-C<params> must be a arrayref of key-value pairs in the order of which the
-values should be returned.
+pairs or a hashref and it will return a list containing only values. The
+C<params> you pass must be a arrayref of key-value pairs. The order of these
+pairs determines the order in which values are returned.
 
 You cannot combine C<slurpy> with C<named_to_list> as there is no way to know
-how the order in which extra values should be returned.
+how to order the extra return values.
 
 =back
 
@@ -197,7 +197,7 @@ a list of values.
 
 For now, you must shift off the invocant yourself.
 
-This subroutine accepts an additional parameter:
+This subroutine accepts the following additional parameters:
 
 =over 4
 
@@ -207,6 +207,9 @@ If this is given, then the generated subroutine will be named using
 L<Sub::Util>. This is strongly recommended as it makes it possible to
 distinguish different check subroutines when profiling or in stack traces.
 
+This name will also be used in some exception messages, even if L<Sub::Util>
+is not available.
+
 Note that you must install L<Sub::Util> yourself separately, as it is not
 required by this distribution, in order to avoid requiring a compiler.
 
@@ -230,10 +233,14 @@ passed to L<Eval::Closure>.
 
 =head1 SUPPORT
 
-Bugs may be submitted through L<https://github.com/houseabsolute/Params-ValidationCompiler/issues>.
+Bugs may be submitted at L<https://github.com/houseabsolute/Params-ValidationCompiler/issues>.
 
 I am also usually active on IRC as 'autarch' on C<irc://irc.perl.org>.
 
+=head1 SOURCE
+
+The source code repository for Params-ValidationCompiler can be found at L<https://github.com/houseabsolute/Params-ValidationCompiler>.
+
 =head1 DONATIONS
 
 If you'd like to thank me for the work I've done on this module, please
@@ -267,16 +274,23 @@ Gregory Oschwald <goschwald@maxmind.com>
 
 =item *
 
+Gregory Oschwald <oschwald@gmail.com>
+
+=item *
+
 Tomasz Konojacki <me@xenu.pl>
 
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by Dave Rolsky.
+This software is Copyright (c) 2016 - 2017 by Dave Rolsky.
 
 This is free software, licensed under:
 
   The Artistic License 2.0 (GPL Compatible)
 
+The full text of the license can be found in the
+F<LICENSE> file included with this distribution.
+
 =cut
