@@ -3,7 +3,7 @@ package Params::ValidationCompiler::Compiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.22';
+our $VERSION = '0.25';
 
 use Carp qw( croak );
 use Eval::Closure qw( eval_closure );
@@ -154,6 +154,12 @@ sub _env { $_[0]->{_env} }
 
 sub named_to_list { $_[0]->{named_to_list} }
 
+sub _inlineable_name {
+    return defined $_[0]->{name}
+        ? $_[0]->{name}
+        : 'an un-named validation subroutine';
+}
+
 sub _any_type_has_coercion {
     my $self = shift;
 
@@ -178,20 +184,25 @@ sub _any_type_has_coercion {
 sub _types {
     my $self = shift;
 
+    my @types;
     if ( ref $self->params eq 'HASH' ) {
-        return map { $_->{type} || () }
+        @types = map { $_->{type} || () }
             grep { ref $_ } values %{ $self->params };
     }
     elsif ( ref $self->params eq 'ARRAY' ) {
         if ( $self->named_to_list ) {
             my %p = @{ $self->params };
-            return map { $_->{type} || () } grep { ref $_ } values %p;
+            @types = map { $_->{type} || () } grep { ref $_ } values %p;
         }
         else {
-            return
-                map { $_->{type} || () } grep { ref $_ } @{ $self->params };
+            @types
+                = map { $_->{type} || () } grep { ref $_ } @{ $self->params };
         }
     }
+
+    push @types, $self->slurpy if $self->slurpy && ref $self->slurpy;
+
+    return @types;
 }
 
 sub subref {
@@ -314,49 +325,54 @@ sub _compile_named_args_check_body {
 sub _set_named_args_hash {
     my $self = shift;
 
-    push @{ $self->_source }, <<'EOF';
-my %args;
-if ( @_ % 2 == 0 ) {
-    %args = @_;
+    push @{ $self->_source },
+        sprintf( <<'EOF', ( $self->_inlineable_name ) x 4 );
+my %%args;
+if ( @_ %% 2 == 0 ) {
+    %%args = @_;
 }
 elsif ( @_ == 1 ) {
     if ( ref $_[0] ) {
         if ( Scalar::Util::blessed( $_[0] ) ) {
             if ( overload::Overloaded( $_[0] )
-                && defined overload::Method( $_[0], '%{}' ) ) {
+                && defined overload::Method( $_[0], '%%{}' ) ) {
 
-                %args = %{ $_[0] };
+                %%args = %%{ $_[0] };
             }
             else {
                 Params::ValidationCompiler::Exception::BadArguments->throw(
                     message =>
-                        'Expected a hash or hash reference but got a single object argument'
+                        'Expected a hash or hash reference but a single object argument was passed to %s',
+                    show_trace => 1,
                 );
             }
         }
         elsif ( ref $_[0] eq 'HASH' ) {
-            %args = %{ $_[0] };
+            %%args = %%{ $_[0] };
         }
         else {
             Params::ValidationCompiler::Exception::BadArguments->throw(
                 message =>
-                    'Expected a hash or hash reference but got a single '
+                    'Expected a hash or hash reference but a single '
                     . ( ref $_[0] )
-                    . ' reference argument',
+                    . ' reference argument was passed to %s',
+                show_trace => 1,
             );
         }
     }
     else {
         Params::ValidationCompiler::Exception::BadArguments->throw(
             message =>
-                'Expected a hash or hash reference but got a single non-reference argument',
+                'Expected a hash or hash reference but a single non-reference argument was passed to %s',
+            show_trace => 1,
         );
     }
 }
 else {
     Params::ValidationCompiler::Exception::BadArguments->throw(
         message =>
-            'Expected a hash or hash reference but got an odd number of arguments',
+            'Expected a hash or hash reference but an odd number of arguments was passed to %s',
+        show_trace => 1,
     );
 }
 EOF
@@ -371,11 +387,12 @@ sub _add_check_for_required_named_param {
 
     my $qname = perlstring($name);
     push @{ $self->_source },
-        sprintf( <<'EOF', $access, ($qname) x 2 );
+        sprintf( <<'EOF', $access, $qname, $self->_inlineable_name, $qname );
 exists %s
     or Params::ValidationCompiler::Exception::Named::Required->throw(
-    message   => %s . ' is a required parameter',
-    parameter => %s,
+    message    => %s . ' is a required parameter for %s',
+    parameter  => %s,
+    show_trace => 1,
     );
 EOF
 
@@ -412,13 +429,14 @@ sub _add_check_for_extra_hash_params {
 
     $self->_env->{'%known'}
         = { map { $_ => 1 } keys %{$params} };
-    push @{ $self->_source }, <<'EOF';
-my @extra = grep { ! $known{$_} } keys %args;
-if ( @extra ) {
+    push @{ $self->_source }, sprintf( <<'EOF', $self->_inlineable_name );
+my @extra = grep { !$known{$_} } keys %%args;
+if (@extra) {
     my $u = join ', ', sort @extra;
     Params::ValidationCompiler::Exception::Named::Extra->throw(
-        message    => "found extra parameters: [$u]",
+        message    => "Found extra parameters passed to %s: [$u]",
         parameters => \@extra,
+        show_trace => 1,
     );
 }
 EOF
@@ -479,6 +497,7 @@ sub _compile_positional_args_check {
         $self->_add_check_for_extra_positional_param_types(
             scalar @specs,
             $self->slurpy,
+            $access_var,
         );
     }
 
@@ -495,7 +514,7 @@ sub _munge_and_check_positional_params {
 
     for my $spec ( @{ $self->params } ) {
         $spec = ref $spec ? $spec : { optional => !$spec };
-        if ( $spec->{optional} ) {
+        if ( $spec->{optional} || exists $spec->{default} ) {
             $in_optional = 1;
         }
         elsif ($in_optional) {
@@ -513,14 +532,16 @@ sub _add_check_for_required_positional_params {
     my $self = shift;
     my $min  = shift;
 
-    push @{ $self->_source }, sprintf( <<'EOF', ($min) x 3 );
+    push @{ $self->_source },
+        sprintf( <<'EOF', ($min) x 2, $self->_inlineable_name, $min );
 if ( @_ < %d ) {
     my $got = scalar @_;
     my $got_n = @_ == 1 ? 'parameter' : 'parameters';
     Params::ValidationCompiler::Exception::Positional::Required->throw(
-        message => "got $got $got_n but expected at least %d",
-        minimum => %d,
-        got     => scalar @_,
+        message    => "Got $got $got_n but expected at least %d for %s",
+        minimum    => %d,
+        got        => scalar @_,
+        show_trace => 1,
     );
 }
 EOF
@@ -529,14 +550,15 @@ EOF
 }
 
 sub _add_check_for_extra_positional_param_types {
-    my $self = shift;
-    my $max  = shift;
-    my $type = shift;
+    my $self       = shift;
+    my $max        = shift;
+    my $type       = shift;
+    my $access_var = shift;
 
     # We need to set the name argument to something that won't conflict with
     # names someone would actually use for a parameter.
     my $check = join q{}, $self->_type_check(
-        '$_[$i]',
+        sprintf( '%s[$i]', $access_var ),
         '__PCC extra parameters__',
         $type,
     );
@@ -555,14 +577,16 @@ sub _add_check_for_extra_positional_params {
     my $self = shift;
     my $max  = shift;
 
-    push @{ $self->_source }, sprintf( <<'EOF', ($max) x 3 );
+    push @{ $self->_source },
+        sprintf( <<'EOF', ($max) x 2, $self->_inlineable_name, $max );
 if ( @_ > %d ) {
     my $extra = @_ - %d;
     my $extra_n = $extra == 1 ? 'parameter' : 'parameters';
     Params::ValidationCompiler::Exception::Positional::Extra->throw(
-        message => "got $extra extra $extra_n",
-        maximum => %d,
-        got     => scalar @_,
+        message    => "Got $extra extra $extra_n for %s",
+        maximum    => %d,
+        got        => scalar @_,
+        show_trace => 1,
     );
 }
 EOF
@@ -897,26 +921,33 @@ Params::ValidationCompiler::Compiler - Object that implements the check subrouti
 
 =head1 VERSION
 
-version 0.22
+version 0.25
 
 =for Pod::Coverage .*
 
 =head1 SUPPORT
 
-Bugs may be submitted through L<https://github.com/houseabsolute/Params-ValidationCompiler/issues>.
+Bugs may be submitted at L<https://github.com/houseabsolute/Params-ValidationCompiler/issues>.
 
 I am also usually active on IRC as 'autarch' on C<irc://irc.perl.org>.
 
+=head1 SOURCE
+
+The source code repository for Params-ValidationCompiler can be found at L<https://github.com/houseabsolute/Params-ValidationCompiler>.
+
 =head1 AUTHOR
 
 Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by Dave Rolsky.
+This software is Copyright (c) 2016 - 2017 by Dave Rolsky.
 
 This is free software, licensed under:
 
   The Artistic License 2.0 (GPL Compatible)
 
+The full text of the license can be found in the
+F<LICENSE> file included with this distribution.
+
 =cut
